import 'package:flutter/cupertino.dart';

import 'package:forme/forme.dart';
import 'package:forme_cupertino_fields/src/field/cupertino_fields.dart';
import 'package:forme_cupertino_fields/src/field/forme_cupertino_slider.dart';
import 'package:forme_cupertino_fields/src/field/forme_cupertino_switch.dart';
import 'package:forme_cupertino_fields/src/forme_cupertino_decorator.dart';

import 'cupertino_segmented_control.dart';

class CupertinoDemoPage extends StatefulWidget {
  @override
  _DemoPageState createState() => _DemoPageState();
}

class _DemoPageState extends State<CupertinoDemoPage> {
  final FormeKey formKey = FormeKey();

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return CupertinoPageScaffold(
      child: SafeArea(
        child: SingleChildScrollView(
          child: Column(children: [
            Padding(
              padding: EdgeInsets.symmetric(horizontal: 20),
              child: createForm(),
            ),
            // for using snackbar here
            Builder(builder: (context) => createButtons(context)),
          ]),
        ),
      ),
    );
  }

  Widget createButtons(BuildContext context) {
    Wrap buttons = Wrap(
      children: [
        Builder(
          builder: (context) {
            return CupertinoButton(
                onPressed: () {
                  formKey.readOnly = !formKey.readOnly;
                  (context as Element).markNeedsBuild();
                },
                child: Text(formKey.readOnly
                    ? 'set form editable'
                    : 'set form readonly'));
          },
        ),
        CupertinoButton(
            onPressed: () {
              setState(() {});
            },
            child: Text('rebuild page')),
        CupertinoButton(
            onPressed: () {
              Map<FormeValueFieldController, String> errorMap =
                  formKey.validate(quietly: true);
              if (errorMap.isNotEmpty) {
                MapEntry<FormeValueFieldController, String> entry =
                    errorMap.entries.first;

                FormeFieldController formeFieldController = entry.key;
                formeFieldController.ensureVisible().then((value) {
                  formeFieldController.requestFocus();
                });
              } else {}
            },
            child: Text('validate')),
        CupertinoButton(
            onPressed: () {
              formKey.reset();
            },
            child: Text('reset')),
        CupertinoButton(
          onPressed: () {
            print(formKey.data);
          },
          child: Text('get form data'),
        ),
      ],
    );
    return buttons;
  }

  Map<String, Widget> createMap(String name) {
    return {
      'A': ReadOnlyWidget(
        text: 'A',
        formeKey: formKey,
        name: name,
      ),
      'B': ReadOnlyWidget(
        text: 'B',
        formeKey: formKey,
        name: name,
      ),
      'C': ValueListenableBuilder2<bool, FormeValidateError?>(
        formKey.lazyFieldListenable(name).readOnlyListenable,
        formKey.lazyFieldListenable(name).errorTextListenable,
        child: Text('C'),
        builder: (context, r, v, child) {
          if (r)
            return Text(
              'C',
              style: TextStyle(color: CupertinoColors.systemGrey),
            );
          if (v == null || !v.hasError) return child!;
          return ShakeWidget(
            child: child!,
            key: UniqueKey(),
          );
        },
      ),
    };
  }

  Widget createForm() {
    Widget child = Column(
      children: [
        CupertinoFormSection.insetGrouped(
            header: Text('Gesture Fields'),
            children: [
              FormeCupertinoSegmentedControl<String>(
                model: FormeCupertinoSegmentedControlModel<String>(
                  padding: const EdgeInsets.symmetric(vertical: 10),
                ),
                decoratorBuilder: FormeCupertinoInputDecoratorBuilder(
                  prefix: Text('control'),
                ),
                name: 'segmentedControl',
                chidren: createMap('segmentedControl'),
                validator: (v) => v == 'C' ? null : 'pls select C',
                autovalidateMode: AutovalidateMode.onUserInteraction,
              ),
              FormeCupertinoSlidingSegmentedControl<String>(
                decoratorBuilder: FormeCupertinoInputDecoratorBuilder(
                    prefix: Text('slidingControl')),
                name: 'segmentedSlidingControl',
                chidren: createMap('segmentedSlidingControl'),
                validator: (v) => v == 'C' ? null : 'pls select C',
                autovalidateMode: AutovalidateMode.onUserInteraction,
              ),
              FormeCupertinoSlider(
                decoratorBuilder: FormeCupertinoInputDecoratorBuilder(
                    prefix: Text('slider'),
                    wrapper: (child) {
                      return Row(
                        children: [
                          Expanded(
                            child: child,
                          )
                        ],
                      );
                    }),
                autovalidateMode: AutovalidateMode.onUserInteraction,
                name: 'slider',
                min: 1,
                max: 100,
                validator: FormeValidates.min(50, errorText: 'value must > 50'),
              ),
              FormeCupertinoPicker(
                  decoratorBuilder: FormeCupertinoInputDecoratorBuilder(
                      prefix: Text('picker'),
                      wrapper: (child) => SizedBox(
                            height: 50,
                            width: double.infinity,
                            child: child,
                          )),
                  validator: (value) => value! < 100
                      ? 'value must bigger than 100,current is $value'
                      : null,
                  name: 'cupertinoPicker',
                  autovalidateMode: AutovalidateMode.onUserInteraction,
                  itemExtent: 30,
                  children: List<Widget>.generate(1000, (index) {
                    return ValueListenableBuilder<bool>(
                        valueListenable: formKey
                            .lazyFieldListenable('cupertinoPicker')
                            .readOnlyListenable,
                        builder: (context, v, child) {
                          if (v)
                            return Text(
                              index.toString(),
                              style:
                                  TextStyle(color: CupertinoColors.systemGrey),
                            );
                          return Text(index.toString());
                        });
                  })),
              FormeCupertinoSwitch(
                decoratorBuilder: FormeCupertinoInputDecoratorBuilder(
                  prefix: Text('switch'),
                ),
                name: 'switch',
                validator: (v) => v! ? null : 'accept and continue',
                autovalidateMode: AutovalidateMode.onUserInteraction,
              ),
            ]),
        CupertinoFormSection.insetGrouped(
          header: const Text('Input Fields'),
          children: [
            FormeCupertinoDateField(
              autovalidateMode: AutovalidateMode.onUserInteraction,
              name: 'dateTime',
              decoratorBuilder:
                  FormeCupertinoInputDecoratorBuilder(prefix: Text('datetime')),
              validator: (value) => value == null ? 'select a date!' : null,
              model: FormeCupertinoDateFieldModel(
                type: FormeDateTimeFieldType.DateTime,
                textFieldModel: FormeCupertinoTextFieldModel(
                  placeholder: 'tap to select',
                ),
              ),
            ),
            FormeCupertinoTimerField(
              autovalidateMode: AutovalidateMode.onUserInteraction,
              name: 'duration',
              decoratorBuilder:
                  FormeCupertinoInputDecoratorBuilder(prefix: Text('duration')),
              validator: (value) => value == null ? 'select a duration!' : null,
              model: FormeCupertinoTimerFieldModel(
                textFieldModel: FormeCupertinoTextFieldModel(
                  placeholder: 'tap to select',
                ),
              ),
            ),
            FormeCupertinoTextField(
                autovalidateMode: AutovalidateMode.onUserInteraction,
                name: 'text',
                decoratorBuilder:
                    FormeCupertinoInputDecoratorBuilder(prefix: Text('text')),
                onValueChanged: (c, v) => print(v),
                validator: FormeValidates.size(
                    min: 10, errorText: 'length must bigger than 10'),
                model: FormeCupertinoTextFieldModel(
                  clearButtonMode: OverlayVisibilityMode.editing,
                )),
            FormeCupertinoNumberField(
                autovalidateMode: AutovalidateMode.onUserInteraction,
                name: 'number',
                decoratorBuilder:
                    FormeCupertinoInputDecoratorBuilder(prefix: Text('number')),
                onValueChanged: (c, v) => print(v),
                validator: FormeValidates.all([
                  FormeValidates.notNull(errorText: 'number can not be empty'),
                  FormeValidates.min(10, errorText: 'must bigger than 10'),
                  FormeValidates.max(100, errorText: 'must smaller than 100')
                ]),
                model: FormeCupertinoNumberFieldModel(
                    textFieldModel: FormeCupertinoTextFieldModel(
                  clearButtonMode: OverlayVisibilityMode.editing,
                ))),
          ],
        ),
        FormeColumn(
          name: 'column',
          children: [],
        ),
      ],
    );
    return Forme(
        child: child,
        key: formKey,
        onValueChanged: (a, b) {
          print(
              '${a.name}\'s value changed ... value: $b ... old value: ${a.oldValue}');
        },
        onErrorChanged: (a, b) {
          print('${a.name} ... error: ${b?.text}');
        },
        onFocusChanged: (a, b) {
          print('${a.name} ... focus: $b');
        });
  }
}
