import 'package:flutter/cupertino.dart';

import 'package:forme/forme.dart';

class FormeCupertinoSlider
    extends ValueField<double, FormeCupertinoSliderModel> {
  FormeCupertinoSlider({
    required String name,
    required double min,
    required double max,
    FormeCupertinoSliderModel? model,
    FormeValueChanged<double, FormeCupertinoSliderModel>? onValueChanged,
    FormFieldValidator<double>? validator,
    AutovalidateMode? autovalidateMode,
    double? initialValue,
    FormFieldSetter<double>? onSaved,
    bool readOnly = false,
    FormeErrorChanged<
            FormeValueFieldController<double, FormeCupertinoSliderModel>>?
        onErrorChanged,
    FormeFocusChanged<
            FormeValueFieldController<double, FormeCupertinoSliderModel>>?
        onFocusChanged,
    FormeFieldInitialed<
            FormeValueFieldController<double, FormeCupertinoSliderModel>>?
        onInitialed,
    Key? key,
    FormeDecoratorBuilder<double>? decoratorBuilder,
  }) : super(
          onInitialed: onInitialed,
          nullValueReplacement: min,
          decoratorBuilder: decoratorBuilder,
          key: key,
          onFocusChanged: onFocusChanged,
          onErrorChanged: onErrorChanged,
          readOnly: readOnly,
          name: name,
          onValueChanged: onValueChanged,
          onSaved: onSaved,
          validator: validator,
          initialValue: initialValue,
          autovalidateMode: autovalidateMode,
          model: (model ?? FormeCupertinoSliderModel())
              .copyWith(FormeCupertinoSliderModel(min: min, max: max)),
          builder: (baseState) {
            _FormeCupertinoSliderState state =
                baseState as _FormeCupertinoSliderState;
            double min = state.model.min!;
            double max = state.model.max!;
            return Focus(
              child: CupertinoSlider(
                  value: state.value!,
                  min: min,
                  max: max,
                  onChangeStart: (v) {
                    state.requestFocus();
                    state.model.onChangeStart?.call(v);
                  },
                  onChangeEnd: (v) {
                    state.didChange(v);
                    state.model.onChangeEnd?.call(v);
                  },
                  activeColor: state.model.activeColor,
                  thumbColor: state.model.thumbColor ?? CupertinoColors.white,
                  divisions: state.model.divisions ?? (max - min).floor(),
                  onChanged: state.readOnly
                      ? null
                      : (v) {
                          state.updateValue(v);
                          state.model.onChanged?.call(v);
                        }),
              focusNode: state.focusNode,
            );
          },
        );

  @override
  _FormeCupertinoSliderState createState() => _FormeCupertinoSliderState();
}

class _FormeCupertinoSliderState
    extends ValueFieldState<double, FormeCupertinoSliderModel> {
  double? _value;

  @override
  double? get value => _value ?? super.value!;

  updateValue(double value) {
    setState(() {
      _value = value;
    });
  }

  @override
  void onValueChanged(double? value) {
    _value = null;
  }

  @override
  void afterUpdateModel(
      FormeCupertinoSliderModel old, FormeCupertinoSliderModel current) {
    if (current.min != null && value! < current.min!) setValue(current.min!);
    if (current.max != null && value! > current.max!) setValue(current.max!);
  }

  @override
  FormeCupertinoSliderModel beforeSetModel(
      FormeCupertinoSliderModel old, FormeCupertinoSliderModel current) {
    if (current.min == null) {
      current = current.copyWith(FormeCupertinoSliderModel(min: old.min));
    }
    if (current.max == null) {
      current = current.copyWith(FormeCupertinoSliderModel(max: old.max));
    }
    return current;
  }
}

class FormeCupertinoSliderModel extends FormeModel {
  final ValueChanged<double>? onChangeStart;
  final ValueChanged<double>? onChangeEnd;
  final ValueChanged<double>? onChanged;
  final double? min;
  final double? max;
  final int? divisions;
  final Color? activeColor;
  final Color? thumbColor;

  FormeCupertinoSliderModel({
    this.onChangeStart,
    this.onChangeEnd,
    this.min,
    this.max,
    this.divisions,
    this.activeColor,
    this.thumbColor,
    this.onChanged,
  });

  @override
  FormeCupertinoSliderModel copyWith(FormeModel oldModel) {
    FormeCupertinoSliderModel old = oldModel as FormeCupertinoSliderModel;

    return FormeCupertinoSliderModel(
      onChangeStart: onChangeStart ?? old.onChangeStart,
      onChangeEnd: onChangeEnd ?? old.onChangeEnd,
      min: min ?? old.min,
      max: max ?? old.max,
      divisions: divisions ?? old.divisions,
      activeColor: activeColor ?? old.activeColor,
      thumbColor: thumbColor ?? old.thumbColor,
      onChanged: onChanged ?? old.onChanged,
    );
  }
}
